<?php
/**
 * @file
 */

/**
 * function for handle gateway response.
 */
function page_get_response_data() {
  $SECURE_SECRET = $_GET['secure_secret'];
  $vpc_txn_secure_hash = $_GET["vpc_SecureHash"];
  unset($_GET["vpc_SecureHash"]);
  // set a flag to indicate if hash has been validated
  $errorexists = FALSE;
  if (strlen($SECURE_SECRET) > 0 && $_GET["vpc_TxnResponseCode"] != "7" && $_GET["vpc_TxnResponseCode"] != "No Value Returned") {
    $md5HashData = $SECURE_SECRET;
    // sort all the incoming vpc response fields and leave out any with no value
    foreach ($_GET as $key => $value) {
      if ($key != "vpc_SecureHash" or strlen($value) > 0) {
        $md5HashData .= $value;
      }
    }
    // Validate the Secure Hash (remember MD5 hashes are not case sensitive)
    // This is just one way of displaying the result of checking the hash.
    // The hash check is all about detecting if the data has changed in transit.
    if (strtoupper($vpc_txn_secure_hash) == strtoupper(md5($md5HashData))) {
      // Secure Hash validation succeeded, add a data field to be displayed
      // later.
      $hashvalidated = "<font color='#00AA00'><strong>CORRECT</strong></FONT>";
    }
    else {
      // Secure Hash validation failed, add a data field to be displayed
      // later.
      $hashvalidated = "<font color='#FF0066'><strong>INVALID HASH</strong></FONT>";
      $errorexists = TRUE;
    }
}
  else {
    // Secure Hash was not validated, add a data field to be displayed later.
    $hashvalidated = "<font color='orange'><strong>Not Calculated - No 'SECURE_SECRET' present.</strong></FONT>";
  }

/**
 * getresponse function.
 */
function getresponsedescription($responsecode) {
  switch ($responsecode) {
    case "0" : $result = "Transaction Successful"; break;
    case "?" : $result = "Transaction status is unknown"; break;
    case "1" : $result = "Unknown Error"; break;
    case "2" : $result = "Bank Declined Transaction"; break;
    case "3" : $result = "No Reply from Bank"; break;
    case "4" : $result = "Expired Card"; break;
    case "5" : $result = "Insufficient funds"; break;
    case "6" : $result = "Error Communicating with Bank"; break;
    case "7" : $result = "Payment Server System Error"; break;
    case "8" : $result = "Transaction Type Not Supported"; break;
    case "9" : $result = "Bank declined transaction (Do not contact Bank)"; break;
    case "A" : $result = "Transaction Aborted"; break;
    case "C" : $result = "Transaction Cancelled"; break;
    case "D" : $result = "Deferred transaction has been received and is awaiting processing"; break;
    case "F" : $result = "3D Secure Authentication failed"; break;
    case "I" : $result = "Card Security Code verification failed"; break;
    case "L" : $result = "Shopping Transaction Locked (Please try the transaction again later)"; break;
    case "N" : $result = "Cardholder is not enrolled in Authentication scheme"; break;
    case "P" : $result = "Transaction has been received by the Payment Adaptor and is being processed"; break;
    case "R" : $result = "Transaction was not processed - Reached limit of retry attempts allowed"; break;
    case "S" : $result = "Duplicate SessionID (OrderInfo)"; break;
    case "T" : $result = "Address Verification Failed"; break;
    case "U" : $result = "Card Security Code Failed"; break;
    case "V" : $result = "Address Verification and Card Security Code Failed"; break;
    default  : $result = "Unable to be determined";
  }
    return $result;
}

// This method uses the verRes status code retrieved from the Digital
// Receipt and returns an appropriate description for the QSI Response Code
// @param statusResponse String containing the 3DS Authentication Status Code
// @return String containing the appropriate description
/**
 * getstatusdescription function.
 */
function getstatusdescription($statusresponse) {
    if ($statusresponse == "" || $statusresponse == "No Value Returned") {
        $result = "3DS not supported or there was no 3DS data provided";
    }
    else {
        switch ($statusresponse) {
            Case "Y"  : $result = "The cardholder was successfully authenticated."; break;
            Case "E"  : $result = "The cardholder is not enrolled."; break;
            Case "N"  : $result = "The cardholder was not verified."; break;
            Case "U"  : $result = "The cardholder's Issuer was unable to authenticate due to some system error at the Issuer."; break;
            Case "F"  : $result = "There was an error in the format of the request from the merchant."; break;
            Case "A"  : $result = "Authentication of your Merchant ID and Password to the ACS Directory Failed."; break;
            Case "D"  : $result = "Error communicating with the Directory Server."; break;
            Case "C"  : $result = "The card type is not supported for authentication."; break;
            Case "S"  : $result = "The signature on the response received from the Issuer could not be validated."; break;
            Case "P"  : $result = "Error parsing input from Issuer."; break;
            Case "I"  : $result = "Internal Payment Server system error."; break;
            default   : $result = "Unable to be determined"; break;
        }
    }
    return $result;
}

/**
 * If input is null, returns string "No Value Returned", else returns input.
 */
function null2unknown($data) {
  if ($data == "") {
    return "No Value Returned";
  }
  else {
    return $data;
  }
}

/** Define Variables
 * ----------------
 * Extract the available receipt fields from the VPC Response
 * If not present then let the value be equal to 'No Value Returned'
 * Standard Receipt Data
 */
  $amount          = null2unknown($_GET["vpc_Amount"]);
  $locale          = null2unknown($_GET["vpc_Locale"]);
  $batchno         = null2unknown($_GET["vpc_BatchNo"]);
  $command         = null2unknown($_GET["vpc_Command"]);
  $message         = null2unknown($_GET["vpc_Message"]);
  $version         = null2unknown($_GET["vpc_Version"]);
  $cardtype        = null2unknown($_GET["vpc_Card"]);
  $orderinfo       = null2unknown($_GET["vpc_OrderInfo"]);
  $receiptno       = null2unknown($_GET["vpc_ReceiptNo"]);
  $merchantid      = null2unknown($_GET["vpc_Merchant"]);
  $authorizeid     = null2unknown($_GET["vpc_AuthorizeId"]);
  $merchtxnref     = null2unknown($_GET["vpc_MerchTxnRef"]);
  $transactionno   = null2unknown($_GET["vpc_TransactionNo"]);
  $acqresponsecode = null2unknown($_GET["vpc_AcqResponseCode"]);
  $txnresponsecode = null2unknown($_GET["vpc_TxnResponseCode"]);

  // 3-D Secure Data
  $vertype         = array_key_exists("vpc_VerType", $_GET)          ? $_GET["vpc_VerType"]          : "No Value Returned";
  $verstatus       = array_key_exists("vpc_VerStatus", $_GET)        ? $_GET["vpc_VerStatus"]        : "No Value Returned";
  $token           = array_key_exists("vpc_VerToken", $_GET)         ? $_GET["vpc_VerToken"]         : "No Value Returned";
  $versecurlevel   = array_key_exists("vpc_VerSecurityLevel", $_GET) ? $_GET["vpc_VerSecurityLevel"] : "No Value Returned";
  $enrolled        = array_key_exists("vpc_3DSenrolled", $_GET)      ? $_GET["vpc_3DSenrolled"]      : "No Value Returned";
  $xid             = array_key_exists("vpc_3DSXID", $_GET)           ? $_GET["vpc_3DSXID"]           : "No Value Returned";
  $acqeci          = array_key_exists("vpc_3DSECI", $_GET)           ? $_GET["vpc_3DSECI"]           : "No Value Returned";
  $authstatus      = array_key_exists("vpc_3DSstatus", $_GET)        ? $_GET["vpc_3DSstatus"]        : "No Value Returned";

  // Show 'Error' in title if an error condition
  $errortxt = "";
  // Show this page as an error page if vpc_TxnResponseCode equals '7'
  if ($txnresponsecode == "7" || $txnresponsecode == "No Value Returned" || $errorexists) {
    $errortxt = "Error ";
  }
    $site_url = $_GET['site_url'];
    $order_id = $_GET['vpc_OrderInfo'];

  //if not an error condition
  if ($txnresponsecode != "7" && $txnresponsecode != "No Value Returned" && $txnresponsecode == "0") {
    $commerce_order = commerce_order_load($order_id);
    $wrapper = entity_metadata_wrapper('commerce_order', $commerce_order);
    $currency = $wrapper->commerce_order_total->currency_code->value();
    $name = 'checkout_complete';
    $order_success = commerce_order_status_update($commerce_order, $name, $skip_save = FALSE, $revision = TRUE, $log = '');
    commerce_checkout_complete($order_success);
    $wrapper = entity_metadata_wrapper('commerce_order', $commerce_order);
    $currency = $wrapper->commerce_order_total->currency_code->value();
    $transaction = commerce_payment_transaction_new('axisbank', $order_id);
    $transaction->amount = $amount;
    $transaction->message = getresponsedescription($txnresponsecode);
    $transaction->currency_code = $currency;
    $transaction->remote_id = $transactionno;
    $transaction->status = COMMERCE_PAYMENT_STATUS_SUCCESS;
    commerce_payment_transaction_save($transaction);
    commerce_payment_redirect_pane_next_page($order_success);
    drupal_goto( $site_url . '/checkout/' . $order_id . '/complete');
  }
  else {
    $commerce_order = commerce_order_load($order_id);
    $name = 'checkout_review';
    $order_failure = commerce_order_status_update($commerce_order, $name, $skip_save = FALSE, $revision = TRUE, $log = '');
    $transaction = commerce_payment_transaction_new('axisbank', $order_id);
    $transaction->message = getresponsedescription($txnresponsecode);
    $transaction->remote_id = $transactionno;
    $transaction->status = COMMERCE_PAYMENT_STATUS_FAILURE;
    commerce_payment_transaction_save($transaction);
    commerce_payment_redirect_pane_previous_page($order_failure);
    drupal_goto( $site_url . '/checkout/' . $order_id . '/review');
  }
}
